#####################################################################################
# Theophylline with ODE model
#####################################################################################

work.dir <- getwd()
save.dir <- file.path(work.dir, "Figures")
save.results <- TRUE # toggle saving results

# Test functions using saemix results
runsaemix <- TRUE

# deSolve
library("deSolve")

#####################################################################################
# Load saemix and run it on theo.saemix to get theo.nocov from which parameters are extracted
if (runsaemix) {
  library("saemix")
  # data
  data("theo.saemix", package = "saemix")
  theo.data <- saemixData(name.data = theo.saemix, header = TRUE, sep = " ", na = NA, 
                          name.group = c("Id"), name.predictors = c("Dose", "Time"), name.response = c("Concentration"), 
                          name.covariates = c("Weight", "Sex"), units = list(x = "hr", y = "mg/L", covariates = c("kg", "-")), 
                          name.X = "Time")
  
  model1cpt <- function(psi, id, xidep) { 
      dose <- xidep[, 1]
      tim <- xidep[, 2]  
      ka <- psi[id, 1]
      V <- psi[id, 2]
      CL <- psi[id, 3]
      k <- CL/V
      ypred <- dose*ka/(V*(ka-k))*(exp(-k*tim)-exp(-ka*tim))
      return(ypred)
  }
  theo.model.base <- saemixModel(model = model1cpt, description = "One-compartment model with first-order absorption", 
                                 psi0 = matrix(c(1., 20, 0.5, 0.1, 0, -0.01), ncol = 3, byrow = TRUE,
                                               dimnames = list(NULL, c("ka", "V", "CL"))), transform.par = c(1, 1, 1))
  
  opt <- list(save = save.results, save.graphs = save.results, directory = file.path(save.dir, "paperTheoNoCov"))
  theo.nocov <- saemix(theo.model.base, theo.data, opt)
}

#####################################################################################

########################
# ad-hoc implementation
# ODE system for 1st order elimination - setting up deSolve
ode1cpt <- function(Time, State, Pars) {
  with(as.list(c(State, Pars)), {
    dGut <- (-ka)*Gut
    dCentral <- ka*Gut - CL/V*Central
    return(list(c(dGut, dCentral)))
  })
}

# testing ode1cpt
par1cpt <- c(ka = 1.53,  # first-order absorption
             V = 31.5,   # volume of central compartment
             CL = 4.1)   # clearance

modelODE1cpt <- function(psi, id, xidep) { 
  ypred <- rep(0, length(id))
  yini <- c(Gut = 0, Central = 0)
  for (isuj in unique(id)) {
    param <- c(ka = psi[isuj, 1], V = psi[isuj, 2], CL = psi[isuj, 3])
    yini[1] <- xidep[match(isuj, id), 1] # Dose
    tim <- c(0, xidep[id == isuj, 2]) # Need to integrate from time = 0
    out <- ode(yini, tim, ode1cpt, param)
    ypred[id == isuj] <- out[-1, 3]/param[2]
  }
  return(ypred)
}

# Initial conditions
yini <- c(Gut = 320, Central = 0)
xtim <- seq(0, 25, by = 0.2)
out1 <- ode(yini, xtim, ode1cpt, par1cpt)

param <- par1cpt
yini[1]*param[1]/(param[2]*(param[1]-param[3]/param[2]))*(exp(-param[3]/param[2]*xtim)-exp(-param[1]*xtim))

# Compare saemix model functions for 1cpt and ODE- should get same results
if (runsaemix) {
  plot(theo.data)
  lines(xtim, out1[, 3]/par1cpt[2], lwd = 3, col = "DarkRed")
  
  xidep <- theo.data@data[, c("Dose", "Time")]
  id <- theo.data@data[, "Id"]
  psi <- exp(theo.nocov@results@phi)
  
  ypred.ode <- modelODE1cpt(psi, id, xidep)
  ypred.cf <- model1cpt(psi, id, xidep)
  
  plot(ypred.ode, ypred.cf)
  abline(0, 1)
  print(summary(ypred.ode-ypred.cf))
  print(summary((ypred.ode-ypred.cf)/(ypred.ode+ypred.cf)*2))
}

# run saemix with deSolve
model.ode1cpt <- saemixModel(model = modelODE1cpt, description = "One-compartment model with first-order absorption, ODE",
                             psi0 = matrix(c(1., 20, 0.5, 0.1, 0, -0.01), ncol = 3, byrow = TRUE,
                                           dimnames = list(NULL, c("ka", "V", "CL"))), transform.par = c(1, 1, 1))
saemix.options <- list(seed = 632545, save = save.results, save.graphs = save.results,
                       directory = file.path(save.dir, "theoODE1cpt"))

# Profiling for speed
Rprof("theoODE1cpt.out")
theoODE.fit <- saemix(model.ode1cpt, theo.data, saemix.options)
Rprof(NULL)

#####################################################################################
