#########################################################################################################
####### Fitting datasets from Plan et al. 2012 with nlme

library("nlme")
prefix.data <- "mc-sim-1-"

# Emax model for nlme/nlmer
nformemax <- ~ exp(le0) + exp(lemax) * input / (exp(le50) + input)
nfunemax <- deriv(nformemax, namevec = c("le0", "lemax", "le50"), 
                  function.arg = c("input", "le0", "lemax", "le50"))

# Initial estimates
pinit <- c(ptrue[c(4, 1:2)], 1, ptrue[c(8, 5, 7)], 1, ptrue[6])
vecinit0 <- c(le0 = log(pinit[1]), lemax = log(pinit[2]), le50 = log(pinit[3]))

for (scenar in scenariosPlan) {
  cat("********************************************************\n")
  cat("        Scenario", scenar, "\n")
  cat("********************************************************\n")
  
  # Datasets
  namdir.data <- file.path(work.dir, "dataPlan12", scenar)
  namres.dir <- paste(scenar, "_nlme", sep = "")
  namdir <- file.path(dir.fit, namres.dir)
  dir.create(namdir)
  resnlme <- resnlme.lscale <- NULL
  
  for (irep in 1:nsim) {
    #    for(irep in 1:2) {
    dat <- read.table(file.path(namdir.data, paste(prefix.data, irep, ".dat", sep = "")), header = TRUE, skip = 1)
    datnlme <- groupedData(DV ~ DOSE | ID, data = dat)
    icount <- 0
    ipb <- 1
    repeat {
      if (icount >= 10 | ipb == 0) break
      icount <- icount + 1
      if (icount == 1) startvecemax <- vecinit0 else startvecemax <- vecinit0 * (1 + rnorm(length(vecinit0), sd = 0.2))
      nlme.fit <- try(nlme(DV ~ nfunemax(DOSE, le0, lemax, le50), data = datnlme,
                           fixed = le0 + lemax + le50 ~ 1, start = startvecemax,
                           random = pdBlocked(list(pdDiag(le0 ~ 1), pdSymm(lemax + le50 ~ 1)))))
      if (class(nlme.fit)[1] != "try-error") ipb <- 0
    } 
    if (ipb == 0) {
      # delta-method to compute SE on parameters (exp(le0), ...)
      vcov.beta <- vcov(nlme.fit)
      beta <- fixef(nlme.fit)
      se.beta <- diag(sqrt(nlme.fit$varFix))
      se.par <- diag(diag(exp(beta)) %*% vcov.beta %*% diag(exp(beta)))
      tab1 <- VarCorr(nlme.fit)
      sd.par <- sqrt(as.double(tab1[, 1]))
      se.omega <- sqrt(as.double(tab1[, 2]))
      
      l1 <- c(irep, beta, sd.par[1:3], NA, sd.par[4], se.beta, se.omega, AIC(nlme.fit), as.double(logLik(nlme.fit)), icount)
      l2 <- c(irep, exp(beta), sd.par[1:3], NA, sd.par[4], se.par, se.omega, AIC(nlme.fit), as.double(logLik(nlme.fit)), icount)
      resnlme.lscale <- rbind(resnlme.lscale, l1)
      resnlme <- rbind(resnlme, l2)
    } else {
      resnlme.lscale <- rbind(resnlme.lscale, c(irep, rep(NA, 18)))
      resnlme <- rbind(resnlme, c(irep, rep(NA, 18)))
    }
  }
  colnames(resnlme) <- c("numsim", "e0", "emax", "e50", "sd.le0", "sd.lemax",
                         "sd.le50", "corr.lemax.le50", "sigma", "se.e0", "se.emax",
                         "se.e50", "se.omegale0", "se.omegalemax", "se.omegale50",
                         "se.sigma", "AIC", "LL", "ntry")
  colnames(resnlme.lscale) <- c("numsim", "le0", "lemax", "le50", "sd.le0", "sd.lemax",
                                "sd.le50", "corr.lemax.le50", "sigma", "se.le0", "se.lemax",
                                "se.le50", "se.omegale0", "se.omegalemax", "se.omegale50",
                                "se.sigma", "AIC", "LL", "ntry")
  # Saving results
  write.table(resnlme.lscale, file.path(namdir, "resnlme_logscale.res"), quote = FALSE, row.names = FALSE)
  write.table(resnlme, file.path(namdir, "resnlme_origscale.res"), quote = FALSE, row.names = FALSE)
}

##################
# Checking delta-method for estimates of SE(parameters) from SE(log-parameters)
library("msm")
deltamethod(list(~ exp(x1), ~ exp(x2), ~ exp(x3)), mean = beta, cov = vcov.beta, ses = FALSE)
diag(exp(beta)) %*% vcov.beta %*% diag(exp(beta))
